#include "dispatch.h"
#include "grid/block.h"

using namespace boost::filesystem;

Dispatch::Dispatch(const string &path)
{
    blockManager_.reset(new GridManager(BlockResolution, 1000));

    loadPCD(path);

    float minX = numeric_limits<float>::max();
    float minY = numeric_limits<float>::max();
    float minZ = numeric_limits<float>::max();
    for(const auto &point : pointcloud_->points){
        minX = minX < point.x ? minX : point.x;
        minY = minY < point.y ? minY : point.y;
        minZ = minZ < point.z ? minZ : point.z;
    }
    offset_ = {minX - 30, minY - 30, minZ};

    for(auto &point : pointcloud_->points){
        point.x -= offset_.x();
        point.y -= offset_.y();
    }
}

bool Dispatch::loadPCD(const string &path)
{
    pointcloud_ = PointCloudExport::Ptr(new PointCloudExport);
    if(exists(path)){
        pcl::io::loadPCDFile(path, *pointcloud_);
        return true;
    }else{
        return false;
    }
}

boost::shared_ptr<GridManager> Dispatch::getBlockManager()
{
    return blockManager_;
}

void Dispatch::dispatch()
{
    if(!pointcloud_){
        LOG(WARNING) << "NO pointcloud_ !";
        return;
    }

//    boost::function<void(const PointExport &)> dispatchFunc
//            = boost::bind(&Dispatch::dispatchPoint, this, _1);
//    multiThreadProcess<PointExport>(pointcloud_, dispatchFunc);

    for(const auto &point : pointcloud_->points){
        dispatchPoint(point);
    }

    auto blockGrids = blockManager_->getGridMap();
    vector<boost::shared_ptr<GridBase>> blockGridVec;
    for(auto iter = blockGrids->begin(); iter != blockGrids->end(); iter++){
        blockGridVec.push_back(iter->second);
    }
    boost::function<void(boost::shared_ptr<GridBase>)> configFunc
            = boost::bind(&Dispatch::configBlockInfo, this, _1);
    syncProcess<boost::shared_ptr<GridBase>>(blockGridVec, configFunc);

//    for(auto iter = blockGrids->begin(); iter != blockGrids->end(); iter++){
//        BlockGrid* block = reinterpret_cast<BlockGrid*>(iter->second.get());
//        block->configInfo();
//    }

//    PointCloudExport::Ptr pointcloud;
//    auto blockGrids = blockManager_->getGridMap();
//    for(auto iter = blockGrids->begin(); iter != blockGrids->end(); iter++){
//        BlockGrid* block = reinterpret_cast<BlockGrid*>(iter->second.get());
//        LOG(INFO) << block->getIndex().transpose();
//        auto gridGrids = block->getGridMap();
//        for(auto it = gridGrids->begin(); it != gridGrids->end(); it++){
//            Grid* grid = reinterpret_cast<Grid*>(it->second.get());
//            PointCloudExport::Ptr pc = grid->getPointCloud();
//            if(pc && pointcloud && pc->size() > pointcloud->size()){
//                pointcloud = pc;
//            }
//        }
//    }
//    if(pointcloud){
//        pcl::io::savePCDFileBinary("/home/juefx/1.pcd", *pointcloud);
    //    }
}

Vector3f Dispatch::getOffset()
{
    return offset_;
}

void Dispatch::dispatchPoint(const PointExport &point)
{
    Vector2i blockIndex = blockManager_->getGridIndexByPoint(point);
    BlockGrid* block = reinterpret_cast<BlockGrid*>(
                blockManager_->getGrid(blockIndex, blockType).get());
    Vector2i gridIndex = block->getGridIndexByPoint(point);

    Grid* grid = reinterpret_cast<Grid*>(block->getGrid(gridIndex).get());
    grid->addPoint(point);

//    Vector2f suppo = block->getCenter() + grid->getCenter();
//    LOG(INFO) << "point: " << point.getArray3fMap().transpose()
//              << " blockIndex: " << blockIndex.transpose()
//              << " gridIndex: " << gridIndex.transpose()
//              << " suppo: " << suppo.transpose();
//    sleep(1);
}

void Dispatch::configBlockInfo(
        boost::shared_ptr<GridBase> blockGrid){
    BlockGrid* block = reinterpret_cast<BlockGrid*>(blockGrid.get());
    block->configInfo();
}
