#include <iostream>
#include <string>
#include <pcl/common/common_headers.h>
#include <pcl/io/pcd_io.h>
#include <pcl/console/parse.h>
#include <pcl/point_types.h>
#include <algorithm>
#include "featureExtraction.h"
using namespace std;

bool featureExtraction::initializeGridFeature(PointCloudExport::Ptr cloud) {
	if (cloud == nullptr || cloud->points.size() == 0) {
		cout << "cloud or ptr of cloud is empty" << endl;
		return false;
	}

	int maxRangeRoad = 0;
	int minRangeRoad = 0;

	int maxRangeNonRoad = 0;
	int minRangeNonRoad = 0;

	int numOfIndexNonRoad = 0;
	int numOfIndexRoad = 0;

	int maxScore = -9999;

	vec_labeledPoint.clear();
	vec_unlabeledPoint.clear();
	vec_labeledPointIndex.clear();
	vec_unlabeledPointIndex.clear();

	for (int i = 0; i < cloud->points.size(); i++) {
		f_xyzil point;
		point.x = cloud->points[i].x;
		point.y = cloud->points[i].y;
		point.z = cloud->points[i].z;
		point.intensity = cloud->points[i].intensity;
		point.label = cloud->points[i].label;
		if (point.label < 20000.1 && point.label>19999.9) {
			vec_labeledPoint.push_back(point);
		}
		else {
			vec_unlabeledPoint.push_back(point);
		}
	}
	setRangeAndIndex();

	toVolumePixelAndSort();
	return true;
}

void featureExtraction::getFeature(vector<gridFeature>& result) {
	if (vec_labeledPoint.size() != 0) {
        gridFeature labeled;
		labeled.minz_devidedByResolution = minRangeRoad / resolutionRoad;
		labeled.deltaz_devidedByResolution = numOfIndexRoad;
		labeled.labeled = true;
		int sum = 0;
		for (int i = 0; i < vec_labeledPoint.size(); i++) {
			sum += vec_labeledPoint[i].intensity;
		}
		labeled.intensity = round((float)sum / (float)vec_labeledPoint.size());
		result.push_back(labeled);
	}
	getNonRoadFeature(result);
}

void featureExtraction::getNonRoadFeature(vector<gridFeature>& result) {
	int n = vec_unlabeledPointIndex.size();
	if (n == 0) {
		return;
	}
	float sum = 0;
	for (int i = 0; i < n; i++) {
		sum += vec_unlabeledPointIndex[i];
	}
	int average = round((float)sum / (float)n);
	int upperBound = average;
	int lowerBound = average;
	for (int i = average; i < numOfIndexNonRoad; i++) {
		for (int j = average; j >= 0; j--) {
			float score = 0;
			for (int k = j; k <= i; k++) {
				if (binarySearch(k)) {
					score += scoreHitHit;
				}
				else {
					score += scoreHitMiss;
				}
			}
			for (int k = 0; k < n; k++) {
				if (vec_unlabeledPointIndex[k] > i || vec_unlabeledPointIndex[k] < j) {
					score += scoreMissHit;
				}
			}
			if (score >= maxScore) {
				maxScore = score;
				upperBound = i;
				lowerBound = j;
			}
			
		}
	}
    gridFeature unlabeled;
	
	unlabeled.minz_devidedByResolution = minRangeNonRoad / resolutionNonRoad + lowerBound;
	unlabeled.deltaz_devidedByResolution = upperBound - lowerBound + 1;
	unlabeled.labeled = false;

	sum = 0;
	for (int i = 0; i < vec_unlabeledPoint.size(); i++) {
		sum += vec_unlabeledPoint[i].intensity;
	}
	unlabeled.intensity = round((float)sum / (float)vec_unlabeledPoint.size());

	result.push_back(unlabeled);



}

bool featureExtraction::binarySearch(int target) {
	int left = 0; int right = vec_unlabeledPointIndex.size() - 1;
	while (left <= right) {
		int mid = (right + left) / 2;
		if (vec_unlabeledPointIndex[mid] == target)
			return true;
		else if (vec_unlabeledPointIndex[mid] < target)
			left = mid + 1;
		else if (vec_unlabeledPointIndex[mid] > target)
			right = mid - 1;
	}
	return false;
}

void featureExtraction::setRangeAndIndex() {
	//Ѱұ߽
	float f_max = -9999.9, f_min = 9999.9;
	for (size_t i = 0; i <vec_labeledPoint.size(); i++) {
		f_max = max(f_max, vec_labeledPoint[i].z);
		f_min = min(f_min, vec_labeledPoint[i].z);
	}
	//趨߽
	maxRangeRoad = ceil(f_max * 100.0 / (float)resolutionRoad) * resolutionRoad;
	minRangeRoad = floor(f_min * 100.0 / (float)resolutionRoad) * resolutionRoad;
	//趨index
	numOfIndexRoad = (maxRangeRoad- minRangeRoad) / resolutionRoad;

	f_max = -9999.9; f_min = 9999.9;
	for (size_t i = 0; i < vec_unlabeledPoint.size(); i++) {
		f_max = max(f_max, vec_unlabeledPoint[i].z);
		f_min = min(f_min, vec_unlabeledPoint[i].z);
	}
	//趨߽
	maxRangeNonRoad = ceil(f_max * 100.0 / (float)resolutionNonRoad) * resolutionNonRoad;
	minRangeNonRoad = floor(f_min * 100.0 / (float)resolutionNonRoad) * resolutionNonRoad;
	//趨index
	numOfIndexNonRoad = (maxRangeNonRoad - minRangeNonRoad) / resolutionNonRoad;



}

void featureExtraction::toVolumePixelAndSort() {
	int z=0;

	for (size_t i = 0; i < vec_labeledPoint.size(); i++) {
		z = floor((vec_labeledPoint[i].z * 100 - (float)minRangeRoad) / (float)resolutionRoad);
		if (z == numOfIndexRoad) {
			z--;
		}
		vec_labeledPointIndex.push_back(z);
	}
	sort(vec_labeledPointIndex.begin(), vec_labeledPointIndex.end());

	vec_labeledPointIndex.erase(unique(vec_labeledPointIndex.begin(), vec_labeledPointIndex.end()), vec_labeledPointIndex.end());



	for (size_t i = 0; i < vec_unlabeledPoint.size(); i++) {
		z = floor((vec_unlabeledPoint[i].z * 100 - (float)minRangeNonRoad) / (float)resolutionNonRoad);
		if (z == numOfIndexNonRoad) {
			z--;
		}
		vec_unlabeledPointIndex.push_back(z);
	}
	sort(vec_unlabeledPointIndex.begin(), vec_unlabeledPointIndex.end());

	vec_unlabeledPointIndex.erase(unique(vec_unlabeledPointIndex.begin(), vec_unlabeledPointIndex.end()), vec_unlabeledPointIndex.end());
}
