#include "gridmanager.h"
#include "block.h"

GridManager::GridManager(
        const float resolution,
        const float sideLength)
    :resolution_(resolution),
      sideLength_(sideLength)
{
    grids_.reset(new map<uint32_t, boost::shared_ptr<GridBase>>);
    itemCntPerSide_ = (uint32_t)(sideLength_ / resolution_);
}

bool GridManager::getGrid(
        const Vector2i &index,
        boost::shared_ptr<GridBase> &grid)
{
    uint32_t encodedIndex = encodeIndex(index);
    auto iter = grids_->find(encodedIndex);
    if(iter != grids_->end()){
        grid = iter->second;
        return true;
    }else{
        return false;
    }
}

boost::shared_ptr<GridBase> GridManager::getGrid(
        const Vector2i &index, const GridType type)
{
    boost::shared_ptr<GridBase> grid;
    uint32_t encodedIndex = encodeIndex(index);
    auto iter = grids_->find(encodedIndex);
    if(iter != grids_->end()){
        grid = iter->second;
    }else{
        if(type == gridType){
            grid = boost::shared_ptr<GridBase>(new Grid(index, resolution_));
        }else if(type == blockType){
            grid = boost::shared_ptr<GridBase>(new BlockGrid(index, resolution_));
        }
        grids_->insert(make_pair(encodedIndex, grid));
    }
    return grid;
}

void GridManager::addGridByIndex(
        const Vector2i &index, const GridType type)
{
    uint32_t encodedIndex = encodeIndex(index);
    boost::shared_ptr<GridBase> grid;
    if(type == gridType){
        grid = boost::shared_ptr<GridBase>(new Grid(index, resolution_));
    }else if(type == blockType){
        grid = boost::shared_ptr<GridBase>(new BlockGrid(index, resolution_));
    }
    boost::unique_lock<boost::shared_mutex> lock(insertMtx_);
    grids_->insert(make_pair(encodedIndex, grid));
}

Vector2i GridManager::getGridIndexByPoint(const PointExport &point)
{
    Vector2i index;
    index.x() = floor((point.x - origin_.x()) / resolution_);
    index.y() = floor((point.y - origin_.y()) / resolution_);
    return index;
}

void GridManager::setOrigin(const Vector2f &origin)
{
    origin_ = origin;
}

const boost::shared_ptr<map<uint32_t, boost::shared_ptr<GridBase> > >
GridManager::getGridMap()
{
    return grids_;
}

float GridManager::getResolution()
{
    return resolution_;
}

void GridManager::setResolution(float resolution)
{
    resolution_ = resolution;
}

void GridManager::setSideLength(float sideLength)
{
    sideLength_ = sideLength;
}

uint32_t GridManager::encodeIndex(const Vector2i &index)
{
    return index.x() * itemCntPerSide_ + index.y();
}
