#ifndef STREAMINFO_H
#define STREAMINFO_H

#include "utils/basic_types.h"
#include "proto_pb/common/header.pb.h"
#include "proto_pb/block/block_index.pb.h"
#include "proto_pb/block/block_info.pb.h"
#include "proto_pb/distribution/distribution.pb.h"
#include <GeographicLib/LocalCartesian.hpp>

typedef pcl::PointCloud<pcl::PointXYZI> PointCloudXYZI;
using namespace GeographicLib;

namespace Internal {
struct BlockDescribe{
    int32_t blockId = 0;
    uint32_t blockOffset = 0;
    uint32_t blockSize = 0;
};
}

typedef ::google::protobuf::RepeatedField< ::google::protobuf::uint32 >
ProtoRepeatedUint32;

class StreamInfo
{
public:
    StreamInfo(const string& streamPath);

    void load();

    void parseOneBlock(
            const Internal::BlockDescribe &internalBlockDescribe);

    vector<uint32_t> parseBitMap(
            const ProtoRepeatedUint32& bitmap,
            const uint32_t size);

    void Forward(double lat, double lon, double h, double& x, double& y, double& z);
private:
    void parseBitmapBlock(
            const BlockInfo& blockInfo,
            const Internal::BlockDescribe &internalBlockDescribe);

    void parseIndexBlock(
            const BlockInfo &blockInfo,
            const Internal::BlockDescribe &internalBlockDescribe);
private:
    string streamPath_;

    uint16_t headerLength_;

    Header header_;

    BlockIndex blockIndex_;

    vector<Internal::BlockDescribe> blockDescribeVec_;

    Distributions Distributions_;

    PointCloudXYZI::Ptr extractedPointcloud_;

    Vector3f offset_ = Vector3f::Zero();

    boost::shared_ptr<LocalCartesian> proj_;
};

#endif // STREAMINFO_H
