//#pragma once
#define PCL_NO_PRECOMPILE
#ifndef PCL_POINT_TYPE_H
#define PCL_POINT_TYPE_H

#include <pcl/point_types.h>
#include <pcl/point_cloud.h>

namespace pcl {
struct PointInternal {
    PCL_ADD_POINT4D
    PCL_ADD_RGB
    float intensity;
    float ring;
    float label = 0;
    float distance;
    double timestamp;
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW  // make sure our new allocators are aligned
} EIGEN_ALIGN16;
}//end namespace pcl

POINT_CLOUD_REGISTER_POINT_STRUCT(
    pcl::PointInternal, (float, x, x)(float, y, y)(float, z, z)(float, rgb, rgb)
        (float, intensity, intensity)(float, ring, ring)(float, label, label)
        (float, distance, distance)(double, timestamp, timestamp))

typedef pcl::PointInternal Point;
typedef pcl::PointCloud<Point> PointCloud;

/////////////////////////////////////////////////////////////
///
///
namespace pcl {
struct PointRos {
    PCL_ADD_POINT4D
    float intensity;
    float ring;
    float label = 0;
    float timestamp;
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW  // make sure our new allocators are aligned
} EIGEN_ALIGN16;
}//end namespace pcl

POINT_CLOUD_REGISTER_POINT_STRUCT(
    pcl::PointRos, (float, x, x)(float, y, y)(float, z, z)
        (float, intensity, intensity)(float, ring, ring)(float, label, label)
        (float, timestamp, timestamp))

typedef pcl::PointRos PointRos;
typedef pcl::PointCloud<PointRos> PointCloudRos;

/////////////////////////////////////////////////////////////
///
///
namespace pcl {
struct PointExport {
    PCL_ADD_POINT4D
    PCL_ADD_RGB
    float intensity = 0;
    float label = 0;
    float info = 0;
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW  // make sure our new allocators are aligned
} EIGEN_ALIGN16;
}//end namespace pcl

POINT_CLOUD_REGISTER_POINT_STRUCT(
    pcl::PointExport, (float, x, x)(float, y, y)(float, z, z)(float, rgb, rgb)
        (float, intensity, intensity)(float, label, label)(float, info, info))

typedef pcl::PointExport PointExport;
typedef pcl::PointCloud<PointExport> PointCloudExport;

/////////////////////////////////////////////////////////////
namespace pcl {
struct PointLabel {
    PCL_ADD_POINT4D
    float intensity;
    float trainLabel = 0;
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW  // make sure our new allocators are aligned
} EIGEN_ALIGN16;
}//end namespace pcl

POINT_CLOUD_REGISTER_POINT_STRUCT(
    pcl::PointLabel, (float, x, x)(float, y, y)(float, z, z)
        (float, intensity, intensity)(float, trainLabel, trainLabel))

typedef pcl::PointLabel PointLabel;
typedef pcl::PointCloud<PointLabel> PointCloudLabel;


#endif // PCL_POINT_TYPE_H
